/*
 *  spkm3/context.c
 *
 *  Copyright (c) 2001 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Andy Adamson <andros@umich.edu>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#ifdef USE_VALICERT
#include <asn1.h>
#include <ssl.h>
#endif /*USE_VALICERT*/
#include <gssapiP_generic.h>
#define OID_EQ 1
#include <gssapi_spkm3.h>
#include <spkm3_mech.h>


/*
* Function:  spkm3_gss_init_context
*
* Description:
* called by spkm3_gss_init_sec_context and accept_sec_context
*  to create and fill the context.
*
* NOTE:  - no creds used.
*/

OM_uint32
spkm3_gss_init_context (
    OM_uint32                *minor_status,
    spkm3_gss_ctx_id_desc    **context_handle_P )
{
  spkm3_gss_ctx_id_desc  *ctx = NULL;


#ifdef DEBUG
    fprintf(stderr, "spkm3_gss_init_context:\n") ;
#endif /* DEBUG */

  ctx = (spkm3_gss_ctx_id_desc*) malloc(sizeof(spkm3_gss_ctx_id_desc)) ;
  if (ctx == NULL) {
    *minor_status = ENOMEM;
    return GSS_S_FAILURE;
  }

  ctx->ctx_id.length =0;
  ctx->ctx_id.data =NULL;
  ctx->target_name = (spkm3_cn_service_t)malloc(sizeof (spkm3_cn_service));
  if (ctx->target_name == NULL) {
    *minor_status = ENOMEM;
    return GSS_S_FAILURE;
  }
  ctx->target_name->length = 0;
  ctx->target_name->data = NULL;
  ctx->source_name = NULL;
  ctx->spkm3_state = SPKM3_REQ_TOKEN; /* do i need these? */
  ctx->share_key = NULL;

  *context_handle_P = ctx;
  minor_status = 0;

  return GSS_S_COMPLETE;
}


/*
* Function: spkm3_construct_target_name()
*
* XXX NOTE: should probably store target name as a subject_dn like 
* name type.
*
* Description:  constructs the X509_NAME target_name using
* the UMICH realm's x509 name preamble, the "nfs" principal
* and the machine dns name and adds the name to the context.
*/

OM_uint32
spkm3_construct_target_name(
    spkm3_gss_ctx_id_desc *ctx,
    spkm3_cn_service_t tname)
{

  int i,len = 0;
#if 0

    for(i=0;i<20;i++){
        if(subject_dn[i].def_val==NULL) break;
        len	+= strlen(subject_dn[i].def_val);
    }
    len += tname->length;
		ctx->target_name->data = (char *)malloc(len);
    if(!ctx->target_name->data) return GSS_S_FAILURE;
    for(i=1;i<20;i++){
        if(subject_dn[i].def_val==NULL) break;
        strncat(ctx->target_name->data,subject_dn[i].def_val,strlen(subject_dn[i].def_val));
    }
		ctx->target_name->data = (char *)malloc(len);
    if(!ctx->target_name->data) return GSS_S_FAILURE;

    strncat(ctx->target_name->data,tname->data,tname->length);
    ctx->target_name->data[ctx->target_name->length] = 0;
    ctx->target_name->length = len;
#endif
/*ANDROS hack*/
    ctx->target_name->data = tname->data;
    ctx->target_name->length = tname->length;
    return GSS_S_COMPLETE;
}
/*
* Function: spkm3_add_targname()
*
* Description:  adds the X509_NAME target_name to the context.
* NOTE: XXX should probably store target name as a subject_dn like 
* name type.
*/

OM_uint32
spkm3_add_targname(
    spkm3_gss_ctx_id_desc *ctx,
    STACK *sk)
{
  int i,err_code = 0;
  X509_NAME_ENTRY *ent = NULL;

#ifdef DEBUG
    fprintf(stderr, "spkm3_add_targname:\n") ;
#endif /* DEBUG */
    for(i=0;i<sk_num(sk);i++){
        ent=(X509_NAME_ENTRY *)sk_value(sk,i);
        ctx->target_name->length += ent->value->length;
    }
    if((ctx->target_name->data = (char *)malloc(ctx->target_name->length+1)) == NULL){
        err_code = ENOMEM;
        goto err;
    }
    for(i=0;i<sk_num(sk);i++){
        ent=(X509_NAME_ENTRY *)sk_value(sk,i);
        strncat(ctx->target_name->data,ent->value->data,ent->value->length);
    }
    ctx->target_name->data[ctx->target_name->length] = 0;
err:
    return err_code;
}

/*
* Function: spkm3_add_keyest2ctx()
*
* Description:  adds the X509_ALGOR key establishment algorithms
* to the context.
* NOTE: currently expect ONLY DH with initiator 1/2 key, with no initiator
* nor target credentials....
*/

OM_uint32
spkm3_add_keyest2ctx(
    spkm3_gss_ctx_id_desc *ctx,
    STACK *keysk)
{
  X509_ALGOR *key;
  int i,nid,err_code = 0;
  long len;
  unsigned char *p;

#ifdef DEBUG
    fprintf(stderr, "spkm3_add_keyest2ctx:\n") ;
#endif /* DEBUG */

    for(i=0;i<sk_num(keysk);i++){
        key = (X509_ALGOR *)sk_value(keysk,i);
        nid = OBJ_obj2nid(key->algorithm);
        printf("spkm3_add_keyest2ctx key algorithm: %s\n",
            (nid == NID_undef)?"UNKNOWN":OBJ_nid2ln(nid));
        if(nid ==NID_dhKeyAgreement) {
            /* dhKeyAgreement */
            printf("NID_dhKeyAgreement: %d\n",NID_dhKeyAgreement);
            p = key->parameter->value.sequence->data;
            len = key->parameter->value.sequence->length;
            if(!d2i_DHparams(&ctx->dh,&p,(long)len))
                goto err;
        }
        else {
            printf("spkm3_add_keyest2ctx: nid %d NOT diffie-hellmen key\n",nid);
            err_code =  EINVAL; /* XXXX */
            goto err;
        }
    }
err:
    return err_code;
}


/*
* Function: spkm3_get_context()
*
* Description: retrieve the spkm3 context described by the 
* context id.
* NOTE: currently only one context held as a local static variable.
* this should change into a cache of contexts
*/

OM_uint32
spkm3_get_context(
    OM_uint32 *min_status,
    spkm3_ctx_id_t *ctx,
    spkm3_gss_ctx_id_desc **context)
{
  static spkm3_gss_ctx_id_desc *spkm3_context = NULL;
  int status = 0;

    if(!spkm3_context){
        if((status = spkm3_gss_init_context(min_status, &spkm3_context)))
            goto err;

    }
/* compare inupt context id with spkm3_context->ctx_id */
    *context = spkm3_context;
    *min_status = 0;
    return GSS_S_COMPLETE;

err:
    *min_status = (OM_uint32) status;
    return GSS_S_FAILURE;
}
