/*
 *  spkm3/gssapi_spkm3.h 
 *
 *  Copyright (c) 2001 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Andy Adamson <andros@umich.edu>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <gssapi/gssapi.h>
#include <spkm_req_rep.h>
/* DH libraries **/
#include<dh.h>
#include<bio.h>
#include<des.h>

/* krb5 declared static oid's */
extern gss_OID gss_nt_user_name;
extern gss_OID gss_nt_machine_uid_name;
extern gss_OID gss_nt_string_uid_name;
extern gss_OID gss_nt_service_name;
extern gss_OID gss_nt_exported_name;

#ifndef OID_EQ
#define g_OID_equal(o1,o2) \
   (((o1)->length == (o2)->length) && \
    ((o1)->elements != 0) && ((o2)->elements != 0) && \
    (memcmp((o1)->elements,(o2)->elements,(int) (o1)->length) == 0))
#define OID_EQ 1
#endif /* OID_EQ */

/* constants */

#define SPKM_TOK_CTX_REQ    0x0100
#define SPKM_TOK_CTX_REP    0x0200
#define SPKM_TOK_CTX_ERROR  0x0300
#define SPKM_TOK_SIGN_MSG   0x0101
#define SPKM_TOK_SEAL_MSG   0x0201
#define SPKM_TOK_MIC_MSG    0x0101
#define SPKM_TOK_WRAP_MSG   0x0201
#define SPKM_TOK_DEL_CTX    0x0102

/* dh stuff - first pass */
#define DES_KEY_SIZE 8

/* billdo's UMICH certificate name preamble (everything but the CN )*/
typedef struct {
  char  *type;
  char  *def_val;
} UMICH_X509_DN;

extern UMICH_X509_DN subject_dn[];

/* structure typedefs */

typedef struct spkm3_ctx_id_t {
    int length;
    unsigned char *data;
}spkm3_ctx_id,*spkm3_ctx_id_t;

/* store full X509 DN as null terminated string 
* in data. i currently concatenate the on the wire
* entries into one string which may change...
*/
typedef struct spkm3_cn_service_t {
    int length;
    char *data;
}spkm3_cn_service,*spkm3_cn_service_t;

/* first pass at spkm3 context. will add a bunch of stuff .... */

typedef struct spkm3_gss_ctx_id_desc_t {
  spkm3_ctx_id         ctx_id; 
  gss_name_t	         source_name ;
  spkm3_cn_service_t	 target_name ;
  OM_uint32			       spkm3_state ;
  OM_uint32			       ret_flags ;
  OM_uint32			       req_flags ;
  DH                   *dh;
  unsigned char        *share_key;
  int                  share_keylen;
} spkm3_gss_ctx_id_desc ;


/* function prototypes */

int i2d_DHPublicKey(DH* a,unsigned char **pp);
DH *d2i_DHPublicKey( DH **a, unsigned char **pp, long length);


void print_dh_sharekey(unsigned char *key, int len);
void print_cn_service(spkm3_cn_service_t name);
void print_req_token(REQ_TOKEN *req);
void print_rep_ti_token(REP_TI_TOKEN *rep);
void print_context_data(Context_Data *cd);
void print_x509_name(X509_NAME *xn);

OM_uint32
spkm3_construct_target_name(
    spkm3_gss_ctx_id_desc *ctx,
    spkm3_cn_service_t tname
);

OM_uint32
spkm3_add_targname(
    spkm3_gss_ctx_id_desc *ctx,
    STACK *sk
);

OM_uint32
spkm3_add_keyest2ctx(
    spkm3_gss_ctx_id_desc *ctx,
    STACK *keysk
);

OM_uint32
spkm3_get_context(
    OM_uint32 *min_status,
    spkm3_ctx_id_t *ctx,
    spkm3_gss_ctx_id_desc **context
);

OM_uint32 
spkm3_gss_init_context (
  OM_uint32  * minor_status,
  spkm3_gss_ctx_id_desc ** context_handle_P
);

OM_uint32
spkm3_gss_context_data(
  Context_Data *reqdata
);

OM_uint32
x509_algor_list_set (
  X509_ALGOR **x,
  int nid
);

OM_uint32
x509_algid_set (
  X509_ALGOR *x);


OM_uint32
x509_algor_dh_set (
  X509_ALGOR **x,
  int nid,
  DH *dh
);
OM_uint32
Conf_Alg_add_entry (
  Conf_Algs *conf_alg,
  X509_ALGOR *alg
);



/* spkm3_gss_ctx state */

#define SPKM3_REQ_TOKEN         0
#define SPKM3_REQ_TI_TOKEN      1
