/*
 *  spkm3/i2d_dh_pu.c
 *
 *  Derived from SSLeay-9.9.0b/crypto/asn1/i2d_r_pu.c
 *
 *  Copyright (c) 2001 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Andy Adamson <andros@umich.edu>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */



#include <stdio.h>
#include <cryptlib.h>
#include <bn.h>
#include <dh.h>
#include <objects.h>
#include <asn1_mac.h>

int i2d_DHPublicKey(a,pp)
DH *a;
unsigned char **pp;
{
  BIGNUM *num[1];
  ASN1_INTEGER bs;
  unsigned int j,i,tot=0,len,max=0,t;
  unsigned char *p;

  if((a == NULL) || (a->pub_key == NULL)) return(0);

  num[0]=a->pub_key;
  for(i=0;i<1;i++)
    {
    j=BN_num_bits(num[i]);
    len=((j == 0)?0:((j/8)+1));
    if (len > max)  max = len;
    len=ASN1_object_size(0,len,
      (num[i]->neg)?V_ASN1_NEG_INTEGER:V_ASN1_INTEGER);
    tot+=len;
    }

  t=ASN1_object_size(1,tot,V_ASN1_SEQUENCE);
  if (pp == NULL) return(t);

  p= *pp;
  ASN1_put_object(&p,1,tot,V_ASN1_SEQUENCE,V_ASN1_UNIVERSAL);

  bs.type=V_ASN1_INTEGER;
  bs.data=(unsigned char *)Malloc(max+4);
  if (bs.data == NULL)
    {
    ASN1err(ASN1_F_I2D_RSAPUBLICKEY,ERR_R_MALLOC_FAILURE);
    return(-1);
    }
 i = 0;
  for (i=0; i<1; i++)
    {
    bs.length=BN_bn2bin(num[i],bs.data);
    i2d_ASN1_INTEGER(&bs,&p);
    }
  Free((char *)bs.data);
  *pp=p;
  return(t);
}
