/*
 *  spkm3/import_name.c
 *
 *  Copyright (c) 2001 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Andy Adamson <andros@umich.edu>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <asn1.h>
#include <ssl.h>
#include <gssapi_spkm3.h>

UMICH_X509_DN subject_dn[] = {
  { "countryName",    "US" } ,
  { "stateOrProvinceName",  "Michigan" } ,
  { "localityName",   "Ann Arbor" } ,
  { "organizationName",   "University of Michigan" } ,
  { "organizationalUnitName", "TEST -- CITI Client CA v1" } ,
  { NULL,       NULL }
};

extern gss_OID gss_nt_service_name;

/*
 * Function: spkm3_gss_import_name()
 *
 * Description: creates the cn name "service/host"
 * NOTE: uses hard coded UMICH preamble.
 */
OM_uint32
spkm3_gss_import_name(
        void *  context,           /* context from mechglue layer currently
                                      not used */
        OM_uint32*  min_status,    /* minor_status */
        gss_buffer_t name_in,      /* input_name_buffer */
        gss_OID  name_oid_in,      /* input_name_type */
        gss_name_t  *name_out )    /* output_name */
{
  spkm3_cn_service    *cn;
  char                *ptr; 

    *name_out = NULL;
    *min_status = 0;
    
    if ((name_oid_in != GSS_C_NULL_OID) &&
       (g_OID_equal(name_oid_in, gss_nt_service_name))){ 

        if((cn = (spkm3_cn_service *)malloc(sizeof(spkm3_cn_service))) == NULL){
            *min_status = ENOMEM;
            return(GSS_S_FAILURE);
        }
        if((cn->data = (char *)malloc(name_in->length + 1)) == NULL){
            *min_status = ENOMEM;
            return(GSS_S_FAILURE);
        }
        memcpy(cn->data, name_in->value, name_in->length);
        cn->data[name_in->length] = 0;

        if ((ptr = strchr(cn->data, '@'))) {
            *ptr = '/';
        }
        cn->length = strlen(cn->data);
        *name_out = (gss_name_t)cn; 
        return 0;
    }
    else { 
        *min_status = EINVAL;
        return(GSS_S_BAD_NAMETYPE);
    }
}
