/*
 * spkm3_mech.c 
 *
 *  Copyright (c) 2001 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Andy Adamson <andros@umich.edu>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <mglueP.h>
#include <spkm3_mech.h>

static struct gss_config spkm3_mechanism =
        {{7, "\052\006\001\005\005\001\003"},
        0,                              /* context, to be filled */
        spkm3_gss_acquire_cred,
        spkm3_gss_release_cred,
        spkm3_gss_init_sec_context,
        spkm3_gss_accept_sec_context,
        spkm3_gss_process_context_token,
        spkm3_gss_delete_sec_context,
        spkm3_gss_context_time,
        spkm3_gss_sign,
        spkm3_gss_verify,
        spkm3_gss_seal,
        spkm3_gss_unseal,
        spkm3_gss_display_status,
        spkm3_gss_indicate_mechs,
        spkm3_gss_compare_name,
        spkm3_gss_display_name,
        spkm3_gss_import_name,
        spkm3_gss_release_name,
        spkm3_gss_inquire_cred,
        spkm3_gss_add_cred,
        spkm3_gss_export_sec_context,
        spkm3_gss_import_sec_context,
        spkm3_gss_inquire_cred_by_mech,
        spkm3_gss_inquire_names_for_mech,
        spkm3_gss_inquire_context,
        spkm3_gss_internal_release_oid,
        spkm3_gss_wrap_size_limit,
        spkm3_pname_to_uid
        };

/* spkm3 code doesn't use the gss_mechanism context */
gss_mechanism
spkm3_gss_initialize()
{
    return (&spkm3_mechanism);
}


/* Function Stubs */
 
OM_uint32
spkm3_gss_acquire_cred (
		    void* context,		/* context */
		    OM_uint32* min_status,		/* minor_status */
		    gss_name_t  name_req,		/* desired_name */
		    OM_uint32  time_req,		/* time_req */
		    gss_OID_set  mech_req,	/* desired_mechs */
		    int cred_cnt,		/* cred_usage */
		    gss_cred_id_t* cred_hndl,	/* output_cred_handle */
		    gss_OID_set*  mech_actual,	/* actual_mechs */
		    OM_uint32*  time_rec		/* time_rec */
		    )
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_release_cred (
		    void*  context,		/* context */		       
		    OM_uint32*  min_status,		/* minor_status */
		    gss_cred_id_t*	cred_hndl/* cred_handle */
)
{
	return GSS_S_FAILURE;
}

#if 0

OM_uint32
spkm3_gss_init_sec_context(
	void*  context,			/* context */
	OM_uint32*  min_status,			/* minor_status */
	gss_cred_id_t  claim_cred_hndl,		/* claimant_cred_handle */
	gss_ctx_id_t*  ctx_hndl,		/* context_handle */
	gss_name_t  name_targ,			/* target_name */
	gss_OID  mech_req,			/* mech_type */
	OM_uint32  flags_req,			/* req_flags */
	OM_uint32  time_req,			/* time_req */
	gss_channel_bindings_t  chan_bnds,	/* input_chan_bindings */
	gss_buffer_t  token_in,		/* input_token */
	gss_OID*  mech_actual,			/* actual_mech_type */
	gss_buffer_t  token_out,		/* output_token */
	OM_uint32*  flags_ret,			/* ret_flags */
	OM_uint32*  time_rec			/* time_rec */
)
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_accept_sec_context (
		    void*  context,			/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_ctx_id_t*  ctx_hndl,		/* context_handle */
		    gss_cred_id_t  cred_hndl,		/* verifier_cred_handle */
		    gss_buffer_t  token_in,		/* input_token_buffer */
		    gss_channel_bindings_t  chan_bnds,	/* input_chan_bindings */
		    gss_name_t*  name_req,		/* src_name */
		    gss_OID*  mech_req,			/* mech_type */
		    gss_buffer_t  token_out,		/* output_token */
		    OM_uint32*  flags_ret,			/* ret_flags */
		    OM_uint32*  time_rec,			/* time_rec */
		    gss_cred_id_t*  del_cred_hndl		/* delegated_cred_handle */
)
{
	return GSS_S_FAILURE;
}

#endif
OM_uint32
spkm3_gss_process_context_token(
		    void* context,		/* context */
		    OM_uint32* min_status,		/* minor_status */
		    gss_ctx_id_t  ctx_hndl,	/* context_handle */
		    gss_buffer_t token_buf	/* token_buffer */
)
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_delete_sec_context(
		    void*  context,		/* context */
		    OM_uint32*  min_status,		/* minor_status */
		    gss_ctx_id_t*  ctx_hndl,	/* context_handle */
		    gss_buffer_t  token_out	/* output_token */
		    )
{
	return GSS_S_FAILURE;
}


OM_uint32
spkm3_gss_context_time(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_ctx_id_t ctx_hndl,	/* context_handle */
		    OM_uint32*  time_rec		/* time_rec */
		    )
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_sign(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_ctx_id_t  ctx_hndl,	/* context_handle */
		    int qop_req,		/* qop_req */
		    gss_buffer_t msg_buf,	/* message_buffer */
		    gss_buffer_t token_buf	/* message_token */
		    )
{
	return 0;
}

OM_uint32
spkm3_gss_verify(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_ctx_id_t  ctx_hndl,	/* context_handle */
		    gss_buffer_t  msg_buf,	/* message_buffer */
		    gss_buffer_t  token_buf,	/* token_buffer */
		    int*  qop_st		/* qop_state */
		    )
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_seal(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_ctx_id_t  ctx_hndl,	/* context_handle */
		    int flag_conf_req,		/* conf_req_flag */
		    int  qop_req,		/* qop_req */
		    gss_buffer_t  msg_in_buf,	/* input_message_buffer */
		    int*  conf_st,		/* conf_state */
		    gss_buffer_t  msg_out_buf	/* output_message_buffer */
		    )
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_unseal(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_ctx_id_t  ctx_hndl,	/* context_handle */
		    gss_buffer_t  msg_in_buf,	/* input_message_buffer */
		    gss_buffer_t  msg_out_buf,	/* output_message_buffer */
		    int*  conf_st,		/* conf_state */
		    int*  qop_st		/* qop_state */
		    )
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_display_status(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    OM_uint32 status_val,		/* status_value */
		    int  status_type,		/* status_type */
		    gss_OID  mech_req,		/* mech_type */
		    OM_uint32* msg_ctx,		/* message_context */
		    gss_buffer_t status_str	/* status_string */
		    )
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_indicate_mechs(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_OID_set*	mech_set/* mech_set */
		    )
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_compare_name(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_name_t  name1,		/* name1 */
		    gss_name_t  name2,		/* name2 */
		    int*  name_equal		/* name_equal */
		    )
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_display_name(
		    void  *context,		/* context */
		    OM_uint32  *min_status,			/* minor_status */
		    gss_name_t name_in,		/* input_name */
		    gss_buffer_t  name_buf_out,	/* output_name_buffer */
		    gss_OID  *name_out		/* output_name_type */
		    )
{
	return GSS_S_FAILURE;
}

#if 0
/* see import_name.c */
OM_uint32
spkm3_gss_import_name(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_buffer_t name_in,	/* input_name_buffer */
		    gss_OID  name_oid_in,		/* input_name_type */
		    gss_name_t  *name_out		/* output_name */
)
{
	return GSS_S_FAILURE;
}
#endif

OM_uint32
spkm3_gss_release_name(
		    void *  context,		/* context */
		    OM_uint32  *min_status,			/* minor_status */
		    gss_name_t  *name		/* input_name */
)
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_inquire_cred(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_cred_id_t crd_hdnl,		/* cred_handle */
		    gss_name_t   *name,		/* name */
		    OM_uint32  *lifetm,		/* lifetime */
		    int   *cred_usage,			/* cred_usage */
		    gss_OID_set  *mech_oid_set		/* mechanisms */
)
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_add_cred(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_cred_id_t  cred_hdnl_in,	/* input_cred_handle */
		    gss_name_t  name_req,		/* desired_name */
		    gss_OID  mech_oid,		/* desired_mech */
		    gss_cred_usage_t  cred_usage,	/* cred_usage */
		    OM_uint32  time_req_init,		/* initiator_time_req */
		    OM_uint32  time_req_accpt,		/* acceptor_time_req */
		    gss_cred_id_t *cred_hdnl_out,	/* output_cred_handle */
		    gss_OID_set   *mech_oid_set,	/* actual_mechs */
		    OM_uint32   *time_rec_init,	/* initiator_time_rec */
		    OM_uint32   *time_rec_accpt	/* acceptor_time_rec */
)
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_export_sec_context(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_ctx_id_t  *ctx_hndl,	/* context_handle */
		    gss_buffer_t	token_inter/* interprocess_token */
)
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_import_sec_context(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_buffer_t  token_inter,	/* interprocess_token */
		    gss_ctx_id_t  *ctx_hndl	/* context_handle */
		    )
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_inquire_cred_by_mech(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_cred_id_t cred_handle,	/* cred_handle */
		    gss_OID  mech_oid,		/* mech_type */
		    gss_name_t   *name,	/* name */
		    OM_uint32   *lifetm_init,	/* initiator_lifetime */
		    OM_uint32   *lifetm_accpt,	/* acceptor_lifetime */
		    gss_cred_usage_t   *cred_usage	/* cred_usage */
)
{
	return GSS_S_FAILURE;
}

#if 0
/* see inquire_names.c */
OM_uint32
spkm3_gss_inquire_names_for_mech(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_OID  mech_oid,		/* mechanism */
		    gss_OID_set*  name_oid_set	/* name_types */
)
{
	return GSS_S_FAILURE;
}
#endif /*0*/

OM_uint32
spkm3_gss_inquire_context(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_ctx_id_t  ctx_hndl,	/* context_handle */
		    gss_name_t * name_src,	/* src_name */
		    gss_name_t * name_targ,	/* targ_name */
		    OM_uint32 *  lifetine_rec,	/* lifetime_rec */
		    gss_OID *  mech_oid,		/* mech_type */
		    OM_uint32 *  flags_ctx,	/* ctx_flags */
		    int *  local_init,           	/* locally_initiated */
		    int *  open		/* open */
)
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_internal_release_oid(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_OID * oid		/* OID */
)
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_gss_wrap_size_limit(
		    void *  context,		/* context */
		    OM_uint32*  min_status,			/* minor_status */
		    gss_ctx_id_t  ctx_hndl,	/* context_handle */
		    int  flog_req,		/* conf_req_flag */
		    gss_qop_t  qop_req,		/* qop_req */
		    OM_uint32  size_req_out,		/* req_output_size */
		    OM_uint32 *  size_max_in		/* max_input_size */
)
{
	return GSS_S_FAILURE;
}

OM_uint32
spkm3_pname_to_uid(
		    void *  context,		/* context */
		    char *  pname,		/* pname */
		    gss_OID  name_oid,		/* name type */
		    gss_OID  mech_oid,		/* mech type */
		    uid_t *	 uid	/* uid */
)
{
	return GSS_S_FAILURE;
}
