/*
 *  spkm3/util.c
 *
 *  Copyright (c) 2001 The Regents of the University of Michigan.
 *  All rights reserved.
 *
 *  Andy Adamson <andros@umich.edu>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#ifdef USE_VALICERT
#include <asn1.h>
#include <ssl.h>
#endif /*USE_VALICERT*/
#include <gssapiP_generic.h>
#define OID_EQ 1
#include <gssapi_spkm3.h>
#include <spkm3_mech.h>

extern const gss_OID_desc *const gss_mech_spkm3;

void
print_dh_sharekey(unsigned char *key, int len)
{
unsigned char *p = key;
int i;
    printf("\nkeylen: %d\n",len);
    for(i=0;i<len;i++){
        printf("%02x",*p++);
    }
    printf("\n");
}

void
print_cn_service(spkm3_cn_service_t name)
{
  int i;
  char *p = name->data;

    for(i=0;i<name->length;i++)
        printf("%c",*p++);
    printf("\n");
}

void
print_algor(X509_ALGOR *alg)
{
int nid;
    nid = OBJ_obj2nid(alg->algorithm);
#if 0
    printf("Algorithm LN %s\n",
        (alg->algorithm->ln)?alg->algorithm->ln:"UNDEFINED");
    printf("Algorithm SN %s\n",
        (alg->algorithm->sn)?alg->algorithm->sn:"UNDEFINED");
#endif /*0*/
    printf("Algorithm: %s\n",
        (nid == NID_undef)?"UNKNOWN":OBJ_nid2ln(nid));
}

void
print_algor_stack(STACK *sk)
{
X509_ALGOR *alg = NULL;
int i,nid;
    for(i=0;i<sk_num(sk);i++){
        alg = (X509_ALGOR *)sk_value(sk,i);
#if 0
        printf("Algorithm LN %s\n",
             (alg->algorithm->ln)?alg->algorithm->ln:"UNDEFINED");
        printf("Algorithm SN %s\n",
             (alg->algorithm->sn)?alg->algorithm->sn:"UNDEFINED");
#endif /*0*/
        nid = OBJ_obj2nid(alg->algorithm);
        printf("Algorithm from NID: %s\n",
            (nid == NID_undef)?"UNKNOWN":OBJ_nid2ln(nid));
    }
}

void
print_context_data(Context_Data *cd)
{

   /* Conf Alg */
    printf("Confidentialty");
    if(cd->confalg->tag == 0)
        print_algor_stack(cd->confalg->d.algs);
    else
        printf("Algorithm tag %d\n", cd->confalg->tag); 

    /* Intg Alg */
    printf("Integrity");
    print_algor_stack(cd->intgalg);
    /* Owf Alg */
    printf("OWF ");
    print_algor_stack(cd->owfalg);
}
void
print_x509_name(X509_NAME *xn)
{
int i,j;
STACK *sk;
X509_NAME_ENTRY *ent = NULL;
char *p;

    sk = xn->entries;
    for(j=0;j<sk_num(sk);j++){
        ent = (X509_NAME_ENTRY *)sk_value(sk,j);
        p = (char *)ent->value->data;
        for(i=0;i<ent->value->length;i++)
            printf("%c",*p++);
        printf("\n");
    }
}
void
print_req_contents(Req_contents *ct)
{

    printf("Req_Contents:\n");
    printf("tokid: %ld\n",ASN1_INTEGER_get(ct->tokid));
/*
    ASNI_BIT_STRING_get_bit is destructive..
    printf("pvno: %d\n", ASN1_BIT_STRING_get_bit(ct->pvno,1));
*/

    printf("target name: ");
    print_x509_name(ct->targname);
    print_context_data(ct->reqdata);
    printf("Key Establishment \n");
    print_algor_stack(ct->keyestbset);
}
void
print_rep_ticontents(Rep_ti_contents *ct)
{

    printf("Rep_ti_contents:\n");
    printf("tokid: %ld\n",ASN1_INTEGER_get(ct->tokid));
    printf("target name: ");
    print_x509_name(ct->targname);
    print_context_data(ct->repdata);
    printf("Key Establishment \n");
    print_algor(ct->keyestbid);
}
void
print_algid(X509_ALGOR *alg)
{
    printf("AlgId:\n");

}
void
print_req_token(REQ_TOKEN *req){

    printf("REQ_TOKEN:\n");
    print_req_contents(req->reqcontents);
    print_algid(req->algId);
}
void
print_rep_ti_token(REP_TI_TOKEN *rep){
    printf("REP_TI_TOKEN:\n");
    print_rep_ticontents(rep->repticontents);
    print_algid(rep->algId);
}
    
